import java.awt.*;
import javax.swing.*;
import java.awt.print.*;
import java.awt.geom.*;

public class PrintUtilities implements Printable {
  private Component componentToBePrinted;
  private double xsize,ysize;
  private Frame fowner;
  private int gotvalues=0;
  private int lo;
  private double shrink;
  
  public static void printComponent(Component c,Frame fr,int x,int y) {
    new PrintUtilities(c,fr,x,y).print();
  }
  
  public PrintUtilities(Component componentToBePrinted,Frame fr,int x,int y) {
    this.componentToBePrinted = componentToBePrinted;
    this.fowner=fr;
    xsize=x;
    ysize=y;
  }
  
  public void print() {
    PrinterJob printJob = PrinterJob.getPrinterJob();
    printJob.setPrintable(this);
    if (printJob.printDialog())
      try {
        printJob.print();
      } catch(PrinterException pe) {
        System.out.println("Error printing: " + pe);
      }
  }

  public int print(Graphics g, PageFormat pageFormat, int pageIndex) {
	double factorx,factory;
	double pox,poy,scy,scx;
	
//	System.out.println("PIndex: "+pageIndex);
	if (pageIndex > 0) {
      return(NO_SUCH_PAGE);
    } else {
      if (gotvalues==0) {
        PrintLayout pl=new PrintLayout(fowner,"Choose Layout");
        pl.pack();
         pl.setVisible(true);
        shrink=pl.getShrink();
        if (shrink<0) return(NO_SUCH_PAGE);
        lo=pl.getChoice();
        if (lo<0) return(NO_SUCH_PAGE);
        pl.dispose();
      }
      gotvalues=1;
//      System.out.println("x: "+pageFormat.getImageableX()+" y: "+pageFormat.getImageableY());
      pox=pageFormat.getImageableX();
      scx=1;
      switch (lo) {
      case 1: {
    	poy=pageFormat.getImageableY();
    	scy=2;
    	break;
      }
      case 2: {
      	poy=pageFormat.getImageableY()+0.5*pageFormat.getImageableHeight();
      	scy=2;
      	break;
        }
      case 3: {
       	poy=pageFormat.getImageableY();
       	scy=3;
       	break;
      }
      case 4: {
      	poy=pageFormat.getImageableY()+0.333333*pageFormat.getImageableHeight();
      	scy=3;
      	break;
        }
      case 5: {
      	poy=pageFormat.getImageableY()+0.666666*pageFormat.getImageableHeight();
      	scy=3;
      	break;
        }
      case 6: {
      	poy=pageFormat.getImageableY();
        scx=2;
    	scy=3;
    	break;
    	  
      }
      case 7: {
      	poy=pageFormat.getImageableY()+0.333333*pageFormat.getImageableHeight();
        scx=2;
    	scy=3;
    	break;
    	  
      }
      case 8: {
        	poy=pageFormat.getImageableY()+0.666666*pageFormat.getImageableHeight();
            scx=2;
        	scy=3;
        	break;
          }
      case 9: {
        	poy=pageFormat.getImageableY();
          	pox=pageFormat.getImageableX()+0.5*pageFormat.getImageableWidth();
            scx=2;
          	scy=3;
          	break;
      }
      case 10: {
        	poy=pageFormat.getImageableY()+0.333333*pageFormat.getImageableHeight();
          	pox=pageFormat.getImageableX()+0.5*pageFormat.getImageableWidth();
            scx=2;
          	scy=3;
          	break;
      }
      case 11: {
      	poy=pageFormat.getImageableY()+0.666666*pageFormat.getImageableHeight();
      	pox=pageFormat.getImageableX()+0.5*pageFormat.getImageableWidth();
        scx=2;
      	scy=3;
      	break;
        }
      
      default : {
        	poy=pageFormat.getImageableY();
        	scy=1;
        }
      }
      Graphics2D g2d = (Graphics2D)g;
      g2d.translate(pox, poy);
      g2d.transform(new AffineTransform(pageFormat.getMatrix()));
      factory=pageFormat.getImageableHeight()/ysize/scy;
      factorx=pageFormat.getImageableWidth()/xsize/scx;
//      System.out.println("lo: "+lo+" pox: "+pox+" poy: "+poy);
//      System.out.println("ph: "+pageFormat.getImageableHeight()+" ysize: "+ysize+" scy: "+scy+" fy: "+factory);
//       System.out.println("pw: "+pageFormat.getImageableWidth()+" xsize: "+xsize+" fx: "+factorx);
      if (factorx>factory) {
    	  factorx=factory;
      }
      g2d.scale(factorx,factorx);
      g2d.scale(shrink,shrink);
      disableDoubleBuffering(componentToBePrinted);
      componentToBePrinted.paint(g2d);
      enableDoubleBuffering(componentToBePrinted);
      return(PAGE_EXISTS);
    }
  }

  public static void disableDoubleBuffering(Component c) {
    RepaintManager currentManager = RepaintManager.currentManager(c);
    currentManager.setDoubleBufferingEnabled(false);
  }

  public static void enableDoubleBuffering(Component c) {
    RepaintManager currentManager = RepaintManager.currentManager(c);
    currentManager.setDoubleBufferingEnabled(true);
  }
}